package org.fuseri.modulecertificate;

import org.fuseri.model.dto.certificate.CertificateCreateDto;
import org.fuseri.model.dto.certificate.CertificateSimpleDto;
import org.fuseri.model.dto.course.CourseDto;
import org.fuseri.model.dto.course.LanguageTypeDto;
import org.fuseri.model.dto.course.ProficiencyLevelDto;
import org.fuseri.model.dto.user.AddressDto;
import org.fuseri.model.dto.user.UserDto;
import org.fuseri.modulecertificate.service.CertificateFacade;
import org.fuseri.modulecertificate.service.CertificateMapper;
import org.fuseri.modulecertificate.service.CertificateService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import java.time.Instant;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@SpringBootTest
@AutoConfigureMockMvc
final class CertificateFacadeTests {
    private final UserDto USER = new UserDto("novakovat",
            "novakova@gamil.com", "Tereza", "Nováková", new AddressDto());
    private final CourseDto COURSE = new CourseDto("A1", 10, LanguageTypeDto.ENGLISH,
            ProficiencyLevelDto.A1);
    private final CertificateCreateDto certificateCreateDto = new CertificateCreateDto(USER, COURSE);
    private final CertificateSimpleDto certificateDto = new CertificateSimpleDto(0L, USER.getId(),
            Instant.now(), COURSE.getId(), "", "");

    private final Certificate certificate = new Certificate();
    private final List<Certificate> certificateList = List.of(certificate);

    @Autowired
    private CertificateFacade certificateFacade;

    @MockBean
    private CertificateService certificateService;

    @MockBean
    private CertificateMapper certificateMapper;

    @Test
    void generateCertificate() {
        when(certificateMapper.mapToCertificate(certificateCreateDto)).thenReturn(certificate);
        when(certificateService.save(certificate)).thenReturn(certificate);
        when(certificateMapper.mapToSimpleDto(certificate)).thenReturn(certificateDto);

        CertificateSimpleDto actualDto = certificateFacade.generate(certificateCreateDto);

        assertEquals(certificateDto, actualDto);
    }

    @Test
    public void testFindById() {
        Long id = 0L;
        when(certificateService.findById(id)).thenReturn(certificate);
        when(certificateMapper.mapToSimpleDto(certificate)).thenReturn(certificateDto);

        CertificateSimpleDto actualDto = certificateFacade.findById(id);

        assertNotNull(actualDto);
        assertEquals(certificateDto, actualDto);
    }

    @Test
    public void testFindForUser() {
        List<CertificateSimpleDto> expectedDtos = List.of(certificateDto);

        when(certificateService.findByUserId(USER.getId())).thenReturn(certificateList);
        when(certificateMapper.mapToList(certificateList)).thenReturn(expectedDtos);

        List<CertificateSimpleDto> actualDtos = certificateFacade.findByUserId(USER.getId());

        assertEquals(expectedDtos, actualDtos);
    }

    @Test
    public void testFindForUserAndCourse() {
        List<CertificateSimpleDto> expectedDtos = List.of(certificateDto);

        when(certificateService.findByUserIdAndCourseId(USER.getId(), COURSE.getId())).thenReturn(certificateList);
        when(certificateMapper.mapToList(certificateList)).thenReturn(expectedDtos);

        List<CertificateSimpleDto> actualDtos = certificateFacade.findByUserIdAndCourseId(USER.getId(), COURSE.getId());

        assertEquals(expectedDtos, actualDtos);
    }

    @Test
    public void testDelete() {
        Long certificateId = 1L;
        certificateFacade.deleteCertificate(certificateId);
        verify(certificateService).delete(certificateId);
    }

    @Test
    public void testFindAll() {
        Pageable pageable = PageRequest.of(0, 10);
        Page<Certificate> certificatePage = new PageImpl<>(List.of(certificate), pageable, 0);
        Page<CertificateSimpleDto> expectedPageDto = new PageImpl<>(List.of(certificateDto), pageable, 0);

        when(certificateService.findAll(pageable)).thenReturn(certificatePage);
        when(certificateMapper.mapToPageDto(certificatePage)).thenReturn(expectedPageDto);

        Page<CertificateSimpleDto> actualPageDto = certificateFacade.findAll(pageable);

        assertEquals(expectedPageDto, actualPageDto);
    }
}
